'use strict';

const deepFreeze = require('deep-freeze-strict');
const AWS = require('aws-sdk');
const Promise = require('bluebird');
const AppErr = require('punch-error');
const ERROR = require('../constants').ERROR;


const ScrapedResult = require('../../scraped-result');

const ERRORS = deepFreeze(
  Object.assign({},
    ERROR,
    {
      MISSING_EVAL: 'Required param eval missing',
      BROKEN_EVAL: 'Eval param doesn\'t have done()',
      MISSING_PROXY: 'Required param proxy missing'
    })
);


const DEFAULTS = deepFreeze({
  LAMBDA_NAME: 'node-phantomjs-aws-lambda-server-development'
});


module.exports.scrape = function(link, config) {
  config = config || {};

  if (!config['proxy']) return Promise.reject(ERRORS.MISSING_PROXY);
  if (!config['strategy']['eval']) return Promise.reject(ERRORS.MISSING_EVAL);
  if (!/done\(.*\)/.test(config['strategy']['eval'])) return Promise.reject(ERRORS.BROKEN_EVAL);
  
  const lambda = new AWS.Lambda({
    accessKeyId: config.strategy.lambda.aws_key,
    secretAccessKey: config.strategy.lambda.aws_secret,
    region: config.strategy.lambda.region
  });

  let params = {
    FunctionName: config.strategy.lambda.lambda_name || DEFAULTS.LAMBDA_NAME,
    Payload : JSON.stringify({
      "proxy": config.proxy.getIP(),
      "eval": config.strategy.eval,
      "options": config.strategy.options,
      "link": link
    })
  };

  return new Promise((resolve, reject) => {
    lambda.invoke(params, function(err, response){
      if (err) {
        console.log(err);
        return reject(new AppErr(ERRORS.LAMBDA_INVOKE_ERROR));
      }
      else if (response.StatusCode !== 200 || response.FunctionError){
        console.log(JSON.parse(response.Payload));
        return reject(new AppErr(ERRORS.LAMBDA_SCRAPER_ERROR));
      }
      resolve(new ScrapedResult(response.Payload));
    });
  });
};
